@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------------
 //
 //  Little Color Management System
-//  Copyright (c) 1998-2017 Marti Maria Saguer
+//  Copyright (c) 1998-2022 Marti Maria Saguer
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the "Software"),
@@ -105,7 +105,7 @@ in the case of future revision (for example, 1994).
 Spatial Resolution: (Two octets) Lightness pixel density in pels/25.4 mm. The basic value is 200. Allowed values are
 100, 200, 300, 400, 600 and 1200 pels/25.4 mm, with square (or equivalent) pels.
 
-NOTE  The functional equivalence of inch-based and mm-based resolutions is maintained. For example, the 200  200
+NOTE - The functional equivalence of inch-based and mm-based resolutions is maintained. For example, the 200 x 200
 */
 
 static
@@ -142,8 +142,8 @@ void SetITUFax(j_compress_ptr cinfo)
 // the default range for ITU/T.42 -- See RFC 2301, section 6.2.3 for details
 
 //  L*  =   [0, 100]
-//  a*  =   [85, 85]
-//  b*  =   [75, 125]
+//  a*  =   [-85, 85]
+//  b*  =   [-75, 125]
 
 
 // These functions does convert the encoding of ITUFAX to floating point
@@ -170,7 +170,7 @@ void Lab2ITU(const cmsCIELab* Lab, cmsUInt16Number Out[3])
 // once for each node. In[] will contain the Lab PCS value to convert to ITUFAX
 // on PCS2ITU, or the ITUFAX value to convert to Lab in ITU2PCS
 // You can change the number of sample points if desired, the algorithm will
-// remain same. 33 points gives good accurancy, but you can reduce to 22 or less
+// remain same. 33 points gives good accuracy, but you can reduce to 22 or less
 // is space is critical
 
 #define GRID_POINTS 33
@@ -272,7 +272,7 @@ cmsHPROFILE CreatePCS2ITU_ICC(void)
 #define PS_FIXED_TO_FLOAT(h, l) ((float) (h) + ((float) (l)/(1<<16)))
 
 static
-cmsBool ProcessPhotoshopAPP13(JOCTET FAR *data, int datalen)
+cmsBool ProcessPhotoshopAPP13(JOCTET *data, int datalen)
 {
     int i;
 
@@ -297,6 +297,8 @@ cmsBool ProcessPhotoshopAPP13(JOCTET FAR *data, int datalen)
         len = ((((GETJOCTET(data[i]<<8) + GETJOCTET(data[i+1]))<<8) +
                          GETJOCTET(data[i+2]))<<8) + GETJOCTET(data[i+3]);
 
+        if (len < 0) return FALSE; // Keep bug hunters away
+
         i += 4; // Size
 
         if (type == 0x03ED && len >= 16) {
@@ -328,7 +330,7 @@ cmsBool HandlePhotoshopAPP13(jpeg_saved_marker_ptr ptr)
 
         if (ptr -> marker == (JPEG_APP0 + 13) && ptr -> data_length > 9)
         {
-            JOCTET FAR* data = ptr -> data;
+            JOCTET* data = ptr -> data;
 
             if(GETJOCTET(data[0]) == 0x50 &&
                GETJOCTET(data[1]) == 0x68 &&
@@ -361,45 +363,62 @@ typedef unsigned int uint32_t;
 #define YRESOLUTION 0x011b
 #define RESOLUTION_UNIT 0x128
 
+// Abort if crafted file
+static
+void craftedFile(void)
+{
+    FatalError("Corrupted EXIF data");
+}
+
 // Read a 16-bit word
 static
-uint16_t read16(uint8_t* arr, int pos,  int swapBytes)
+uint16_t read16(uint8_t* arr, size_t pos,  int swapBytes, size_t max)
 {
-    uint8_t b1 = arr[pos];
-    uint8_t b2 = arr[pos+1];
+    if (pos + 2 >= max)
+    {
+        craftedFile();
+        return 0;
+    }
+    else
+    {
+        uint8_t b1 = arr[pos];
+        uint8_t b2 = arr[pos + 1];
 
-    return (swapBytes) ?  ((b2 << 8) | b1) : ((b1 << 8) | b2);
+        return (swapBytes) ? ((b2 << 8) | b1) : ((b1 << 8) | b2);
+    }
 }
 
 
 // Read a 32-bit word
 static
-uint32_t read32(uint8_t* arr, int pos,  int swapBytes)
+uint32_t read32(uint8_t* arr, size_t pos, int swapBytes, size_t max)
 {
 
-    if(!swapBytes) {
-
-        return (arr[pos]   << 24) |
-               (arr[pos+1] << 16) |
-               (arr[pos+2] << 8) |
-                arr[pos+3];
+    if (pos + 4 >= max)
+    {
+        craftedFile();
+        return 0;
     }
+    else
+    {
+        if (!swapBytes) {
 
-    return arr[pos] |
-           (arr[pos+1] << 8) |
-           (arr[pos+2] << 16) |
-           (arr[pos+3] << 24);
+            return (arr[pos] << 24) | (arr[pos + 1] << 16) | (arr[pos + 2] << 8) | arr[pos + 3];
+        }
+
+        return arr[pos] | (arr[pos + 1] << 8) | (arr[pos + 2] << 16) | (arr[pos + 3] << 24); 
+    }
 }
 
 
 
 static
-int read_tag(uint8_t* arr, int pos,  int swapBytes, void* dest)
+int read_tag(uint8_t* arr, int pos,  int swapBytes, void* dest, size_t max)
 {
         // Format should be 5 over here (rational)
-    uint32_t format = read16(arr, pos + 2, swapBytes);
+    uint32_t format = read16(arr, pos + 2, swapBytes, max);
     // Components should be 1
-    uint32_t components = read32(arr, pos + 4, swapBytes);
+    uint32_t components = read32(arr, pos + 4, swapBytes, max);
     // Points to the value
     uint32_t offset;
 
@@ -409,20 +428,20 @@ int read_tag(uint8_t* arr, int pos,  int swapBytes, void* dest)
     if (format == 3)
         offset = pos + 8;
     else
-        offset =  read32(arr, pos + 8, swapBytes);
+        offset =  read32(arr, pos + 8, swapBytes, max);
 
     switch (format) {
 
     case 5: // Rational
           {
-          double num = read32(arr, offset, swapBytes);
-          double den = read32(arr, offset + 4, swapBytes);
+          double num = read32(arr, offset, swapBytes, max);
+          double den = read32(arr, offset + 4, swapBytes, max);
           *(double *) dest = num / den;
           }
           break;
 
     case 3: // uint 16
-        *(int*) dest = read16(arr, offset, swapBytes);
+        *(int*) dest = read16(arr, offset, swapBytes, max);
         break;
 
     default:  return 0;
@@ -435,7 +454,7 @@ int read_tag(uint8_t* arr, int pos,  int swapBytes, void* dest)
 
 // Handler for EXIF data
 static
-    cmsBool HandleEXIF(struct jpeg_decompress_struct* cinfo)
+cmsBool HandleEXIF(struct jpeg_decompress_struct* cinfo)
 {
     jpeg_saved_marker_ptr ptr;
     uint32_t ifd_ofs;
@@ -443,12 +462,14 @@ static
     uint32_t i, numEntries;
     double XRes = -1, YRes = -1;
     int Unit = 2; // Inches
-
+    
 
     for (ptr = cinfo ->marker_list; ptr; ptr = ptr ->next) {
 
         if ((ptr ->marker == JPEG_APP0+1) && ptr ->data_length > 6) {
-            JOCTET FAR* data = ptr -> data;
+
+            JOCTET* data = ptr -> data;
+            size_t max = ptr->data_length;
 
             if (memcmp(data, "Exif\0\0", 6) == 0) {
 
@@ -457,7 +478,7 @@ static
                 // 8 byte TIFF header
                 // first two determine byte order
                 pos = 0;
-                if (read16(data, pos, 0) == INTEL_BYTE_ORDER) {
+                if (read16(data, pos, 0, max) == INTEL_BYTE_ORDER) {
                     swapBytes = 1;
                 }
 
@@ -467,28 +488,28 @@ static
                 pos += 2;
 
                 // offset to Image File Directory (includes the previous 8 bytes)
-                ifd_ofs = read32(data, pos, swapBytes);
+                ifd_ofs = read32(data, pos, swapBytes, max);
 
                 // Search the directory for resolution tags
-                numEntries = read16(data, ifd_ofs, swapBytes);
+                numEntries = read16(data, ifd_ofs, swapBytes, max);
 
                 for (i=0; i < numEntries; i++) {
 
                     uint32_t entryOffset = ifd_ofs + 2 + (12 * i);
-                    uint32_t tag = read16(data, entryOffset, swapBytes);
+                    uint32_t tag = read16(data, entryOffset, swapBytes, max);
 
                     switch (tag) {
 
                     case RESOLUTION_UNIT:
-                        if (!read_tag(data, entryOffset, swapBytes, &Unit)) return FALSE;
+                        if (!read_tag(data, entryOffset, swapBytes, &Unit, max)) return FALSE;
                         break;
 
                     case XRESOLUTION:
-                        if (!read_tag(data, entryOffset, swapBytes, &XRes)) return FALSE;
+                        if (!read_tag(data, entryOffset, swapBytes, &XRes, max)) return FALSE;
                         break;
 
                     case YRESOLUTION:
-                        if (!read_tag(data, entryOffset, swapBytes, &YRes)) return FALSE;
+                        if (!read_tag(data, entryOffset, swapBytes, &YRes, max)) return FALSE;
                         break;
 
                     default:;
@@ -864,8 +885,8 @@ int DoTransform(cmsHTRANSFORM hXForm, int OutputColorSpace)
        if (EmbedProfile && cOutProf)
            DoEmbedProfile(cOutProf);
 
-       ScanLineIn  = (JSAMPROW) malloc(Decompressor.output_width * Decompressor.num_components);
-       ScanLineOut = (JSAMPROW) malloc(Compressor.image_width * Compressor.num_components);
+       ScanLineIn  = (JSAMPROW) malloc((size_t) Decompressor.output_width * Decompressor.num_components);
+       ScanLineOut = (JSAMPROW) malloc((size_t) Compressor.image_width * Compressor.num_components);
 
        while (Decompressor.output_scanline <
                             Decompressor.output_height) {
@@ -1026,82 +1047,63 @@ int TransformImage(char *cDefInpProf, char *cOutputProf)
 }
 
 
-// Simply print help
-
 static
 void Help(int level)
 {
-     fprintf(stderr, "little cms ICC profile applier for JPEG - v3.2 [LittleCMS %2.2f]\n\n", LCMS_VERSION / 1000.0);
 
-     switch(level) {
+    UTILS_UNUSED_PARAMETER(level);
 
-     default:
-     case 0:
+    fprintf(stderr, "usage: jpgicc [flags] input.jpg output.jpg\n");
 
-     fprintf(stderr, "usage: jpgicc [flags] input.jpg output.jpg\n");
+    fprintf(stderr, "\nflags:\n\n");
+    fprintf(stderr, "-v - Verbose\n");
+    fprintf(stderr, "-i<profile> - Input profile (defaults to sRGB)\n");
+    fprintf(stderr, "-o<profile> - Output profile (defaults to sRGB)\n");
 
-     fprintf(stderr, "\nflags:\n\n");
-     fprintf(stderr, "%cv - Verbose\n", SW);
-     fprintf(stderr, "%ci<profile> - Input profile (defaults to sRGB)\n", SW);
-     fprintf(stderr, "%co<profile> - Output profile (defaults to sRGB)\n", SW);
+    PrintBuiltins();
 
-     PrintRenderingIntents();
+    PrintRenderingIntents();
 
 
-     fprintf(stderr, "%cb - Black point compensation\n", SW);
-     fprintf(stderr, "%cd<0..1> - Observer adaptation state (abs.col. only)\n", SW);
-     fprintf(stderr, "%cn - Ignore embedded profile\n", SW);
-     fprintf(stderr, "%ce - Embed destination profile\n", SW);
-     fprintf(stderr, "%cs<new profile> - Save embedded profile as <new profile>\n", SW);
+    fprintf(stderr, "-b - Black point compensation\n");
+    fprintf(stderr, "-d<0..1> - Observer adaptation state (abs.col. only)\n");
+    fprintf(stderr, "-n - Ignore embedded profile\n");
+    fprintf(stderr, "-e - Embed destination profile\n");
+    fprintf(stderr, "-s<new profile> - Save embedded profile as <new profile>\n");
 
-     fprintf(stderr, "\n");
+    fprintf(stderr, "\n");
 
-     fprintf(stderr, "%cc<0,1,2,3> - Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes) [defaults to 1]\n", SW);
-     fprintf(stderr, "\n");
+    fprintf(stderr, "-c<0,1,2,3> - Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes) [defaults to 1]\n");
+    fprintf(stderr, "\n");
 
-     fprintf(stderr, "%cp<profile> - Soft proof profile\n", SW);
-     fprintf(stderr, "%cm<0,1,2,3> - SoftProof intent\n", SW);
-     fprintf(stderr, "%cg - Marks out-of-gamut colors on softproof\n", SW);
-     fprintf(stderr, "%c!<r>,<g>,<b> - Out-of-gamut marker channel values\n", SW);
+    fprintf(stderr, "-p<profile> - Soft proof profile\n");
+    fprintf(stderr, "-m<0,1,2,3> - SoftProof intent\n");
+    fprintf(stderr, "-g - Marks out-of-gamut colors on softproof\n");
+    fprintf(stderr, "-!<r>,<g>,<b> - Out-of-gamut marker channel values\n");
 
-     fprintf(stderr, "\n");
-     fprintf(stderr, "%cq<0..100> - Output JPEG quality\n", SW);
+    fprintf(stderr, "\n");
+    fprintf(stderr, "-q<0..100> - Output JPEG quality\n");
 
-     fprintf(stderr, "\n");
-     fprintf(stderr, "%ch<0,1,2,3> - More help\n", SW);
-     break;
+    fprintf(stderr, "Examples:\n\n"
+        "To color correct from scanner to sRGB:\n"
+        "\tjpgicc -iscanner.icm in.jpg out.jpg\n"
+        "To convert from monitor1 to monitor2:\n"
+        "\tjpgicc -imon1.icm -omon2.icm in.jpg out.jpg\n"
+        "To make a CMYK separation:\n"
+        "\tjpgicc -oprinter.icm inrgb.jpg outcmyk.jpg\n"
+        "To recover sRGB from a CMYK separation:\n"
+        "\tjpgicc -iprinter.icm incmyk.jpg outrgb.jpg\n"
+        "To convert from CIELab ITU/Fax JPEG to sRGB\n"
+        "\tjpgicc in.jpg out.jpg\n\n");
 
-     case 1:
 
-     fprintf(stderr, "Examples:\n\n"
-                     "To color correct from scanner to sRGB:\n"
-                     "\tjpgicc %ciscanner.icm in.jpg out.jpg\n"
-                     "To convert from monitor1 to monitor2:\n"
-                     "\tjpgicc %cimon1.icm %comon2.icm in.jpg out.jpg\n"
-                     "To make a CMYK separation:\n"
-                     "\tjpgicc %coprinter.icm inrgb.jpg outcmyk.jpg\n"
-                     "To recover sRGB from a CMYK separation:\n"
-                     "\tjpgicc %ciprinter.icm incmyk.jpg outrgb.jpg\n"
-                     "To convert from CIELab ITU/Fax JPEG to sRGB\n"
-                     "\tjpgicc in.jpg out.jpg\n\n",
-                     SW, SW, SW, SW, SW);
-     break;
+    fprintf(stderr, "This program is intended to be a demo of the Little CMS\n"
+        "color engine. Both lcms and this program are open source.\n"
+        "You can obtain both in source code at https://www.littlecms.com\n"
+        "For suggestions, comments, bug reports etc. send mail to\n"
+        "info@littlecms.com\n\n");
 
-     case 2:
-         PrintBuiltins();
-         break;
-
-     case 3:
-
-     fprintf(stderr, "This program is intended to be a demo of the little cms\n"
-                     "engine. Both lcms and this program are freeware. You can\n"
-                     "obtain both in source code at http://www.littlecms.com\n"
-                     "For suggestions, comments, bug reports etc. send mail to\n"
-                     "marti@littlecms.com\n\n");
-     break;
-     }
-
-     exit(0);
+    exit(0);
 }
 
 
@@ -1112,11 +1114,22 @@ void HandleSwitches(int argc, char *argv[])
 {
     int s;
 
-    while ((s=xgetopt(argc,argv,"bBnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:Q:q:M:m:L:l:eEs:S:!:D:d:")) != EOF) {
+    while ((s=xgetopt(argc,argv,"bBnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:Q:q:M:m:L:l:eEs:S:!:D:d:-:")) != EOF) {
 
         switch (s)
         {
 
+        case '-':
+            if (strcmp(xoptarg, "help") == 0)
+            {
+                Help(0);
+            }
+            else
+            {
+                FatalError("Unknown option - run without args to see valid ones.\n");
+            }
+            break;
+
         case 'b':
         case 'B':
             BlackPointCompensation = TRUE;
@@ -1236,6 +1249,11 @@ void HandleSwitches(int argc, char *argv[])
 
 int main(int argc, char* argv[])
 {
+
+    fprintf(stderr, "Little CMS ICC profile applier for JPEG - v3.3 [LittleCMS %2.2f]\n\n", LCMS_VERSION / 1000.0);
+    fprintf(stderr, "Copyright (c) 1998-2022 Marti Maria Saguer. See COPYING file for details.\n");
+    fflush(stderr);
+
     InitUtils("jpgicc");
 
     HandleSwitches(argc, argv);
